// main.h
//=======================================================================
const char version[64] = "\nJed Margolin's Flash Memory Demo   7/10/2024";

#define LCD_E 10
#define LCD_RS 11
#define LCD_D4 6
#define LCD_D5 7
#define LCD_D6 8
#define LCD_D7 9

#define PB1 14
#define PB2 13
#define PB3 12
#define SW 15
#define CHANNEL 3    // RF Channel
#define LED1 2
#define LED2 27

#define TXEN 21    // nrf905 Transmit Enable
#define TRX_CE 20  // enable nrf905 chip for transmit/receive
#define AM 28      // Address Match
#define DR 22      // Data Ready
#define CSN905 17  // nrf906 CSN - pin 22
#define BUF_LEN  33

#define UART_TX_PIN 0
#define UART_RX_PIN 1
#define UART_ID uart0
#define BAUD_RATE 115200

#define ADC0_PIN 26

#define REGMODE 23    // voltage regulator mode  1 = quiet mode

extern struct myswitch switch1;
extern struct myswitch switch2;
extern struct myswitch switch3;

extern void do_switch(unsigned char, struct myswitch *); 
extern unsigned char timer_flag;

// WC1[0] is the instruction and the address start for nrf905
const unsigned char WC1[11] = { 0x00, 0x70, 0x0C, 0x44, 0x20, 0x20, 0x61, 0x62, 0x63, 0x64, 0xDB };  // 433.6 MHz
const unsigned char TXADR1[5] = { 0x22, 0x61, 0x62, 0x63, 0x64 };  // TXADR[0] is the instruction

const unsigned char WC2[11] = { 0x00, 0x6E, 0x0C, 0x44, 0x20, 0x20, 0x54, 0x53, 0x52, 0x51, 0xDB };  // 433.4 MHz
const unsigned char TXADR2[5] = { 0x22, 0x54, 0x53, 0x52, 0x51 };  // TXADR[0] is the instruction

unsigned char spi_out_buffer[BUF_LEN];   // out_buffer[0] is the instruction
unsigned char spi_in_buffer[BUF_LEN];    // in_buffer[0] is the status byte

void nrf905_init(void);

void do_flash(void);
extern void do_clock(void);
void do_adc(void);

// Messages (duplicate in clock.h)
struct messages
{
    unsigned char pos;        // position
    unsigned char numchars;   // number of characters
    unsigned char msg[];      // message
};

uint8_t out_buffer[10];   // first byte is register address
//   i2c_write_blocking(i2c_default, DSADR, out_buffer, nbytes_out, false);

extern unsigned char nflag;
struct repeating_timer timer;
//=======================================================
char nstr[512];
char line[512];
char start_date[20];

unsigned char mode;   // 0 = RX; 1 = clock

unsigned char irq_flag, secs_flag, uart_flag;
unsigned int save_count, irq_count; 
unsigned char nflag;
unsigned int ts_number;
uint32_t adc_avg;
uint16_t adc_log_flag, adc_log_count; 

void LED_break(void);
void my_itoa(unsigned int, char *);
void my_itoa2(unsigned int, char *);
void line_fill_bin(unsigned char,char *, unsigned char, unsigned char);
void line_fill(unsigned char, char *, unsigned char, unsigned char);
void log_time(void);
void log2_time_uart(void);  //&&
//===============================================================
//Time-stamped log 32 bytes 
struct log
{
   uint16_t ts_number;     // index number                   0,1
   
   uint8_t ts_date;       // ASCII time-stamped date        2
   uint8_t ts_day;        // ASCII time-stamped day         3
   uint8_t ts_month;      // ASCII time-stamped month       4
   uint8_t ts_year;       // ASCII time-stamped year        5

   uint8_t ts_hours;      // ASCII time-stamped hour        6
   uint8_t ts_minutes;    // ASCII time-stamped minutes     7
   uint8_t ts_seconds;    // ASCII time-stamped second      8

   uint16_t adc_val;      //                                9, 10

   uint8_t data[20];      // just data                      11 - 30                   

   uint8_t flag;          // flag for non-blank             31
};

void make_adc_entry(void);
struct log flash_entry;   // pump log in RAM, 32 log entries
uint8_t test_data_start;
void show_log(struct log *ptr);
//================================================================
// LCD Messages
const struct messages tx_msg1 = {0x81, 14, "Jed Margolin's"};
const struct messages tx_msg2 = {0xC2, 10,  "Flash Demo"};
const struct messages tx_msg3 = {0x83, 8,  "(C) 2024"};
const struct messages tx_msg4 = {0x82, 12, "Flash 7/10/24"};
//=================================================================
// From clock.c
#define SECS   0
#define MINS   1
#define HOURS  2
#define DAY    3
#define DATE   4
#define MONTH  5
#define YEAR   6

extern void i2c_get_time(void);
extern unsigned char time_buffer[];
extern const char month_array[18][3];
extern const struct messages msg1, msg2, msg3,msg15;
extern const char *month_name[12];
extern const char day_array[7][3];

extern void SetChrMode(void);
extern void SetCmdMode(void);
extern void lcd_init(void);
extern void domsg(const struct messages *);
extern void lcd_wr_data(unsigned char);
extern void lcd_wr_instr(unsigned char);
extern void lcd_text(char *);
extern void lcd_byte(char);
extern void lcd_wr_hex(unsigned int, unsigned char);
extern void convert_hex_bcd2(unsigned char, unsigned char *);
extern void lcd_wr_bcd(unsigned int, unsigned char);
extern void lcd_wr_bcd2(unsigned int, unsigned char);
extern void lcd_wr_bcd_A(unsigned int, unsigned char);
extern void lcd_wr_time(unsigned int, unsigned int);
extern void lcd_wr_hex16(unsigned int, unsigned int);
extern void lcd_wr_bcd4(unsigned int, unsigned int);
extern void clear_lcd(unsigned char);
//================================================================
#define SECTOR_MAX 16  // sectors per block
#define FLASH_TARGET_OFFSET (512 * 1024)  // start of my flash data
#define FLASH_SDATE_SECTOR 258     // 256 sectors for my log, 16 sectors per page
#define FLASH_SDATE_PAGE 4128     

void do_flash_test(void);
void write_flash(unsigned char *, uint32_t);
void write_flash_sector(void);
void copy_ram_log_to_flash(struct log *ptr);
void erase_block(uint16_t);
void erase_all_blocks(void);
uint16_t is_sector_blank(uint32_t);
void read_flash_uart(uint32_t sector);
void read_sector(uint16_t sector, uint16_t number);
void do_uart_RX(void);
void log4_data_uart(struct log *);
void write_log(struct log *);
void erase_all_sectors(void);
void log1_time_uart(void);
void show_log2(struct log *);

void write_adc_log(struct log *);

uint16_t is_page_blank(uint32_t);    
void read_page(uint16_t, uint16_t);
void find_first_blank_page(void);  // Find the first blank Page_Index (amd Bock_index)
void myflash_erase_sector(uint16_t);
void erase_sector(uint16_t);

uint16_t first_time_flash;

void dump_flash(uint32_t, uint32_t, uint16_t);
void dump_flash_page(uint32_t, uint16_t);
void dump_flash_page_hex(uint32_t); 
void write_page(uint32_t, unsigned char *);
uint32_t ints;  // for interrupt
unsigned int Entry_Read_Index, Flash_Write_Index, erase_flag, reset_rx_start;
uint16_t Page_Index, Last_Page, Sector_Index;
unsigned char uart_flag_RX, uart_char;
void do_uart_RX(void);

void test_read_sector(uint16_t);
void test_write_sector(uint16_t);
//================================================
void do_start_date(uint16_t);
void show_page(uint32_t);
void my_hextoa2(unsigned char, char *);
unsigned char temp_buffer[256];
void dump_flash_page(uint32_t, uint16_t);  // with or without also dumping hex
void copy_ram_log_to_flash(struct log *);
void log_flash_uart(uint32_t);
void show_log_ptr(struct log *);
void show_log_page(uint32_t);
//===============================================
/*  RPI Pico sizes
Name 	            C++ 	Bytes 	Range

bool 		                    1 	true or false
signed char 	    int8_t 	    1 	-128 to 127
unsigned char 	    uint8_t 	1 	0 to 255
short int 	        int16_t 	2 	-32768 to 32767
unsigned short int 	uint16_t 	2 	0 to 65535
int 	            int32_t 	4 	-2147483648 to 2147483647
unsigned int 	    uint32_t 	4 	0 to 4294967295
long int 	        int32_t 	4 	-2147483648 to 2147483647
unsigned long int 	uint32_t 	4 	0 to 4294967295
long long 	        int64_t 	8 	−9,223,372,036,854,775,808 to 9,223,372,036,854,775,807
unsigned long long 	uint64_t 	8 	0 to 18,446,744,073,709,551,615
float 		                    4 	+/- 3.4e +/- 38 (~7 digits)
double 		                    8 	+/- 1.7e +/- 308 (~15 digits)
long double 		            8 	+/- 1.7e +/- 308 (~15 digits)
wchar_t 		            2 or 4  1 wide character
*/

